@@ -1,10 +1,3 @@
-use std::sync::Arc;
-
-use {
-    ahash::AHashMap,
-    rusttype::{point, Scale},
-};
-
 use crate::{
     mutex::{Mutex, RwLock},
     text::{
@@ -13,7 +6,10 @@ use crate::{
     },
     TextureAtlas,
 };
+use ahash::AHashMap;
 use emath::{vec2, Vec2};
+use std::collections::BTreeSet;
+use std::sync::Arc;
 
 // ----------------------------------------------------------------------------
 
@@ -32,7 +28,7 @@ pub struct UvRect {
 
 #[derive(Clone, Copy, Debug)]
 pub struct GlyphInfo {
-    id: rusttype::GlyphId,
+    id: ab_glyph::GlyphId,
 
     /// Unit: points.
     pub advance_width: f32,
@@ -44,7 +40,7 @@ pub struct GlyphInfo {
 impl Default for GlyphInfo {
     fn default() -> Self {
         Self {
-            id: rusttype::GlyphId(0),
+            id: ab_glyph::GlyphId(0),
             advance_width: 0.0,
             uv_rect: None,
         }
@@ -56,7 +52,7 @@ impl Default for GlyphInfo {
 /// A specific font with a size.
 /// The interface uses points as the unit for everything.
 pub struct FontImpl {
-    rusttype_font: Arc<rusttype::Font<'static>>,
+    ab_glyph_font: ab_glyph::FontArc,
     /// Maximum character height
     scale_in_pixels: f32,
     height_in_points: f32,
@@ -71,7 +67,7 @@ impl FontImpl {
     pub fn new(
         atlas: Arc<Mutex<TextureAtlas>>,
         pixels_per_point: f32,
-        rusttype_font: Arc<rusttype::Font<'static>>,
+        ab_glyph_font: ab_glyph::FontArc,
         scale_in_points: f32,
         y_offset: f32,
     ) -> FontImpl {
@@ -80,14 +76,23 @@ impl FontImpl {
 
         let scale_in_pixels = pixels_per_point * scale_in_points;
 
+        // Round to an even number of physical pixels to get even kerning.
+        // See https://github.com/emilk/egui/issues/382
+        let scale_in_pixels = scale_in_pixels.round();
+        let scale_in_points = scale_in_pixels / pixels_per_point;
+
         let height_in_points = scale_in_points;
+
         // TODO: use v_metrics for line spacing ?
         // let v = rusttype_font.v_metrics(Scale::uniform(scale_in_pixels));
         // let height_in_pixels = v.ascent - v.descent + v.line_gap;
         // let height_in_points = height_in_pixels / pixels_per_point;
 
+        // Round to closest pixel:
+        let y_offset = (y_offset * pixels_per_point).round() / pixels_per_point;
+
         Self {
-            rusttype_font,
+            ab_glyph_font,
             scale_in_pixels,
             height_in_points,
             y_offset,
@@ -97,6 +102,24 @@ impl FontImpl {
         }
     }
 
+    /// An un-ordered iterator over all supported characters.
+    fn characters(&self) -> impl Iterator<Item = char> + '_ {
+        use ab_glyph::Font as _;
+        self.ab_glyph_font
+            .codepoint_ids()
+            .map(|(_, chr)| chr)
+            .filter(|chr| {
+                !matches!(
+                    chr,
+                    // Strip out a religious symbol with secondary nefarious interpretation:
+                    '\u{534d}' | '\u{5350}' |
+
+                    // Ignore ubuntu-specific stuff in `Ubuntu-Light.ttf`:
+                    '\u{E0FF}' | '\u{EFFD}' | '\u{F0FF}' | '\u{F200}'
+                )
+            })
+    }
+
     /// `\n` will result in `None`
     fn glyph_info(&self, c: char) -> Option<GlyphInfo> {
         {
@@ -106,8 +129,9 @@ impl FontImpl {
         }
 
         // Add new character:
-        let glyph = self.rusttype_font.glyph(c);
-        if glyph.id().0 == 0 {
+        use ab_glyph::Font as _;
+        let glyph_id = self.ab_glyph_font.glyph_id(c);
+        if glyph_id.0 == 0 {
             if invisible_char(c) {
                 // hack
                 let glyph_info = GlyphInfo::default();
@@ -119,7 +143,8 @@ impl FontImpl {
         } else {
             let mut glyph_info = allocate_glyph(
                 &mut self.atlas.lock(),
-                glyph,
+                &self.ab_glyph_font,
+                glyph_id,
                 self.scale_in_pixels,
                 self.y_offset,
                 self.pixels_per_point,
@@ -138,12 +163,13 @@ impl FontImpl {
 
     pub fn pair_kerning(
         &self,
-        last_glyph_id: rusttype::GlyphId,
-        glyph_id: rusttype::GlyphId,
+        last_glyph_id: ab_glyph::GlyphId,
+        glyph_id: ab_glyph::GlyphId,
     ) -> f32 {
-        let scale_in_pixels = Scale::uniform(self.scale_in_pixels);
-        self.rusttype_font
-            .pair_kerning(scale_in_pixels, last_glyph_id, glyph_id)
+        use ab_glyph::{Font as _, ScaleFont};
+        self.ab_glyph_font
+            .as_scaled(self.scale_in_pixels)
+            .kern(last_glyph_id, glyph_id)
             / self.pixels_per_point
     }
 
@@ -166,6 +192,7 @@ type FontIndex = usize;
 pub struct Font {
     text_style: TextStyle,
     fonts: Vec<Arc<FontImpl>>,
+    characters: std::collections::BTreeSet<char>,
     replacement_glyph: (FontIndex, GlyphInfo),
     pixels_per_point: f32,
     row_height: f32,
@@ -174,10 +201,16 @@ pub struct Font {
 
 impl Font {
     pub fn new(text_style: TextStyle, fonts: Vec<Arc<FontImpl>>) -> Self {
+        let mut characters = BTreeSet::new();
+        for font in &fonts {
+            characters.extend(font.characters());
+        }
+
         if fonts.is_empty() {
             return Self {
                 text_style,
                 fonts,
+                characters,
                 replacement_glyph: Default::default(),
                 pixels_per_point: 0.0,
                 row_height: 0.0,
@@ -191,6 +224,7 @@ impl Font {
         let mut slf = Self {
             text_style,
             fonts,
+            characters,
             replacement_glyph: Default::default(),
             pixels_per_point,
             row_height,
@@ -223,6 +257,11 @@ impl Font {
         slf
     }
 
+    /// All supported characters
+    pub fn characters(&self) -> &BTreeSet<char> {
+        &self.characters
+    }
+
     #[inline(always)]
     pub fn text_style(&self) -> TextStyle {
         self.text_style
@@ -335,20 +374,23 @@ impl Font {
         self.finalize_galley(galley)
     }
 
-    /// Always returns at least one row.
     /// Will line break at `\n`.
+    ///
+    /// Always returns at least one row.
     pub fn layout_no_wrap(&self, text: String) -> Galley {
         self.layout_multiline(text, f32::INFINITY)
     }
 
+    /// Will wrap text at the given width and line break at `\n`.
+    ///
     /// Always returns at least one row.
-    /// Will wrap text at the given width.
     pub fn layout_multiline(&self, text: String, max_width_in_points: f32) -> Galley {
         self.layout_multiline_with_indentation_and_max_width(text, 0.0, max_width_in_points)
     }
 
     /// * `first_row_indentation`: extra space before the very first character (in points).
     /// * `max_width_in_points`: wrapping width.
+    ///
     /// Always returns at least one row.
     pub fn layout_multiline_with_indentation_and_max_width(
         &self,
@@ -388,7 +430,8 @@ impl Font {
                 row.y_max += cursor_y;
             }
             cursor_y = paragraph_rows.last().unwrap().y_max;
-            cursor_y += row_height * 0.2; // Extra spacing between paragraphs. TODO: less hacky
+
+            // cursor_y += row_height * 0.2; // Extra spacing between paragraphs.
 
             rows.append(&mut paragraph_rows);
 
@@ -461,7 +504,7 @@ impl Font {
         let mut out_rows = vec![];
 
         for (i, (x, chr)) in full_x_offsets.iter().skip(1).zip(text.chars()).enumerate() {
-            debug_assert!(chr != '\n');
+            crate::epaint_assert!(chr != '\n');
             let potential_row_width = first_row_indentation + x - row_start_x;
 
             if potential_row_width > max_width_in_points {
@@ -605,20 +648,22 @@ fn invisible_char(c: char) -> bool {
 
 fn allocate_glyph(
     atlas: &mut TextureAtlas,
-    glyph: rusttype::Glyph<'static>,
+    font: &ab_glyph::FontArc,
+    glyph_id: ab_glyph::GlyphId,
     scale_in_pixels: f32,
     y_offset: f32,
     pixels_per_point: f32,
 ) -> GlyphInfo {
-    assert!(glyph.id().0 != 0);
+    assert!(glyph_id.0 != 0);
+    use ab_glyph::{Font as _, ScaleFont};
 
-    let glyph = glyph.scaled(Scale::uniform(scale_in_pixels));
-    let glyph = glyph.positioned(point(0.0, 0.0));
+    let glyph =
+        glyph_id.with_scale_and_position(scale_in_pixels, ab_glyph::Point { x: 0.0, y: 0.0 });
 
-    let uv_rect = if let Some(bb) = glyph.pixel_bounding_box() {
+    let uv_rect = font.outline_glyph(glyph).and_then(|glyph| {
+        let bb = glyph.px_bounds();
         let glyph_width = bb.width() as usize;
         let glyph_height = bb.height() as usize;
-
         if glyph_width == 0 || glyph_height == 0 {
             None
         } else {
@@ -645,15 +690,13 @@ fn allocate_glyph(
                 ),
             })
         }
-    } else {
-        // No bounding box. Maybe a space?
-        None
-    };
+    });
 
-    let advance_width_in_points = glyph.unpositioned().h_metrics().advance_width / pixels_per_point;
+    let advance_width_in_points =
+        font.as_scaled(scale_in_pixels).h_advance(glyph_id) / pixels_per_point;
 
     GlyphInfo {
-        id: glyph.id(),
+        id: glyph_id,
         advance_width: advance_width_in_points,
         uv_rect,
     }
